##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
"""
XML DOM Builder
"""
from Shared.DC.xml import pyexpat
from OFS import ZDOM
import Globals

class ParseError(Exception): pass

class Builder:
    """
    DOM Builder base class
    
    'nodes' maps Node type codes to classes
    override this class attribute in subclasses
    """
    
    nodes={}
    
    buffer_size=1028 * 32

    def __init__(self, doc):
        self.doc=doc
        self.reset(doc)

    # Public methods
    # --------------
    
    def reset(self, root=None):
        """
        Reset the builder.
        root is the element to root the tree in.
        """
        if root is None:
            root=self.nodes[ZDOM.DOCUMENT_FRAGMENT_NODE](self.doc)
        self.cur_elem = self.root = root
        self.error = None
        self.in_cdata=0

    def parse(self, input):
        """
        Build a DOM tree give a string or a fileobject.
        """
        p = pyexpat.ParserCreate()
        p.StartElementHandler = self.start
        p.EndElementHandler = self.end
        p.CharacterDataHandler = self.cdata
        p.StartCdataSectionHandler = self.start_cdata
        p.EndCdataSectionHandler = self.end_cdata
        p.ProcessingInstructionHandler = self.pi
        p.CommentHandler = self.comment
        p.StartNamespaceDeclHandler = self.start_ns
        p.EndNamespaceDeclHandler = self.end_ns
     
        if type(input)==type(''):
            rv = p.Parse(input, 1)
        else:
            while 1:
                if Globals.DatabaseVersion == '3':
                   get_transaction().commit(1)
                v=input.read(self.buffer_size)
                if v == "":
                    rv = 1
                    break
                rv = p.Parse(v, 0)
                if not rv:
                    break
        if not rv:
            raise ParseError('%s at line %s' % 
                    (pyexpat.ErrorString(p.ErrorCode), p.ErrorLineNumber))
        else:
            self.root.normalize()
            return self.root
            
    # Handler methods
    # ---------------

    def start(self, name, attrs):
        e=self.nodes[ZDOM.ELEMENT_NODE](self.doc, name)      
        if self.cur_elem is not None:
            self.cur_elem.appendChild(e)
            self.cur_elem = e.__of__(self.cur_elem)
        else:
            self.cur_elem = self.root = e
        for i in range(0, len(attrs) ,2):
            self.cur_elem.setAttribute(attrs[i], attrs[i+1])
             
    def end(self, name):
        try: self.cur_elem=self.cur_elem.aq_parent      
        except: pass
    
    def cdata(self, data):
        if self.in_cdata:
            e=self.nodes[ZDOM.CDATA_SECTION_NODE](self.doc, data)
        else:
            e=self.nodes[ZDOM.TEXT_NODE](self.doc, data)
        self.cur_elem.appendChild(e)
    
    def pi(self, target, data):
        e=self.nodes[ZDOM.PROCESSING_INSTRUCTION_NODE](self.doc, target, data)
        self.cur_elem.appendChild(e)
    
    def comment(self, data):
        e=self.nodes[ZDOM.COMMENT_NODE](self.doc, data)
        self.cur_elem.appendChild(e)
    
    def start_cdata(self):
        self.in_cdata=1
        
    def end_cdata(self):
        self.in_cdata=0
    
    def start_ns(self, prefix, uri):
        pass
        
    def end_ns(self, prefix):
        pass
    
