##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
"""
Python Product for XML Documents
"""

import Node
from Node import Text, Comment, ProcessingInstruction, CDATASection, \
    DocumentFragment
from Globals import HTMLFile, MessageDialog
from DateTime import DateTime
import string
from urllib import quote
import Builder
from OFS import ZDOM

class Manageable:
    """
    Mix-in for Zope management
    """

    isPrincipiaFolderish=1
                             
    manage_options=({'label':'Edit', 'action':'manage_editForm'},
                    {'label':'Upload', 'action':'manage_uploadForm'},
                    {'label':'Properties', 'action':'manage_propertiesForm'},
                    {'label':'Security',   'action':'manage_access'},
                    )
                    
    __ac_permissions__=(
    ('Access contents information',
        ('objectIds', 'objectValues', 'objectItems', 'getfirstChild', 'getlastChild',
         'getNextSibling', 'getPreviousSibling', 'getChildNodes', 'getParentNode',
         'getOwnerDocument', 'getElementsByTagName', 'nextObject', 'previousObject',
         'objectIndex'),
        ('Anonymous', 'Manager')),
    ('View management screens',
        ('manage', 'manage_main', 'manage_editForm', 'manage_uploadForm')),
    ('Change XML Documents',
        ('manage_edit', 'manage_upload', 'insertBefore', 'appendChild',
         'removeChild', 'cloneNode', 'normalize', 'setAttribute',
         'setAttributeNode', 'removeAttribute', 'removeAttributeNode',
         'parse', 'PUT', 'createElement', 'createTextNode', 'createCDATASection',
         'createComment', 'createProcessingInstruction', 'createDocumentFragment'
         )),
    ('View',
        ('', 'text_content', 'getnodeType', 'getnodeValue', 'toXML',
         'document_src', 'getAttribute', 'getAttributes')),
    ('Manage properties', 
        ('manage_addProperty', 'manage_editProperties', 'manage_delProperties',
         'manage_changeProperties',)),
    ('FTP access', ('manage_FTPstat','manage_FTPget','manage_FTPlist')),
    )

    # Content Editing Methods
    # -----------------------
    
    manage_uploadForm=HTMLFile('documentUpload', globals())

    def get_size(self): return len(self.toXML())

    _size_changes={
        'Bigger': (5,5),
        'Smaller': (-5,-5),
        'Narrower': (0,-5),
        'Wider': (0,5),
        'Taller': (5,0),
        'Shorter': (-5,0),
        }

    def _er(self,data,title,SUBMIT,dtpref_cols,dtpref_rows,REQUEST):
        dr,dc = self._size_changes[SUBMIT]
        rows=max(1,string.atoi(dtpref_rows)+dr)
        cols=max(40,string.atoi(dtpref_cols)+dc)
        e=(DateTime('GMT') + 365).rfc822()
        resp=REQUEST['RESPONSE']
        resp.setCookie('dtpref_rows',str(rows),path='/',expires=e)
        resp.setCookie('dtpref_cols',str(cols),path='/',expires=e)
        return self.manage_editForm(
            self,REQUEST,title=title,toXML=data,
            dtpref_cols=cols,dtpref_rows=rows)

    def manage_edit(self,data,title=None,SUBMIT='Change',dtpref_cols='50',
                    dtpref_rows='20',REQUEST=None):
        """
        Replaces a Documents contents with Data, Title with Title.

        The SUBMIT parameter is also used to change the size of the editing
        area on the default Document edit screen.  If the value is "Smaller",
        the rows and columns decrease by 5.  If the value is "Bigger", the
        rows and columns increase by 5.  If any other or no value is supplied,
        the data gets checked for DTML errors and is saved.
        """
        if self._size_changes.has_key(SUBMIT):
            return self._er(data,title,SUBMIT,dtpref_cols,dtpref_rows,REQUEST)
        if title is not None:
            self.title=title
        try:
            self.parse(data)
        except Builder.ParseError, message:
            get_transaction().abort()
            if REQUEST:
                return self.manage_editForm(self, REQUEST, toXML=data,
                        manage_tabs_message='<font color="red">XML Parsing Error: %s</font>' % message)
        if REQUEST:
            message="Content changed."
            return self.manage_editForm(self, REQUEST, manage_tabs_message=message)

    def manage_upload(self, file='', REQUEST=None):
        """Replace the contents of the document with the text in file."""  
        try:
            self.parse(file)
        except Builder.ParseError, message:
            get_transaction().abort()
            if REQUEST: return MessageDialog(
                    title  ='XML Parsing Error',
                    message=message,
            action ='manage_workspace') 
        if REQUEST: return MessageDialog(
                    title  ='Success!',
                    message='Your changes have been saved',
                    action ='manage_workspace')

    def manage_afterAdd(self, item, container): pass
    def manage_beforeDelete(self, item, container): pass
    def manage_afterClone(self, item): pass
    

# Node classes
# ------------

class Document(Manageable, Node.Document):
    """
    XML Document Zope object
    """

    meta_type='XML Document'

    manage=manage_main=manage_editForm=HTMLFile('documentEdit', globals())

    def _get_builder(self):
        # returns a DOM builder 
        return DOMBuilder(self)
        
    def _fragment(self, data):
        # Return a Document fragment given XML data.
        p=self._get_builder()
        p.reset()
        p.parse(data)
        return p.root
   
    def parse(self, data):
        """
        Build a DOM tree rooted in this node given XML text or file object.
        """
        self._clear()
        p=self._get_builder()
        p.parse(data)

    # DOM write methods
    # -----------------
    
    def createElement(self, tagName):
        """
        Creates an element of the type specified. Note that the instance
        returned implements the Element interface, so attributes can be
        specified directly on the returned object. 
        """
        return self._get_builder().nodes[ZDOM.ELEMENT_NODE](self, tagName)
        
    def createTextNode(self, data):
        """
        Creates a Text node given the specified string. 
        """
        return self._get_builder().nodes[ZDOM.TEXT_NODE](self, data)
    
    def createCDATASection(self, data):
        """
        Creates a CDATASection node whose value is the specified string. 
        """
        return self._get_builder().nodes[ZDOM.CDATA_SECTION_NODE](self, data)
        
    def createComment(self, data):
        """
        Creates a Comment node given the specified string. 
        """
        return self._get_builder().nodes[ZDOM.COMMENT_NODE](self, data)

    def createProcessingInstruction(self, target, data):
        """
        Creates a ProcessingInstruction node given the specified
        name and data strings. 
        """
        return self._get_builder().nodes[ZDOM.PROCESSING_INSTRUCTION_NODE](self, data)
         
    def createAttribute(self, name):
        """
        Creates an Attr of the given name. Note that the Attr instance can then
        be set on an Element using the setAttribute method. 
        """
        return self._get_builder().nodes[ZDOM.ATTRIBUTE_NODE](self, name, '')
       
    def createDocumentFragment(self):
        """
        Creates an empty DocumentFragment object.
        """
        return self._get_builder().nodes[ZDOM.DOCUMENT_FRAGMENT_NODE]()
       
    # Protocol handlers
    # -----------------
     
    def PUT(self, REQUEST, RESPONSE):
        """Handle HTTP PUT requests"""
        self.dav__init(REQUEST, RESPONSE)
        body=REQUEST.get('BODY', '')   
        self.parse(body)
        RESPONSE.setStatus(204)
        return RESPONSE        
     
    def manage_FTPget(self):
        "Get source for FTP download"
        return self.toXML()
     
     
class Element(Manageable, Node.Element):
    """
    XML Element Zope object
    """

    icon="misc_/XMLDocument/xml_element.gif"

    def parse(self, data):
        """
        Build a DOM tree rooted in this node given XML text or file object.
        """
        e=self.aq_acquire('_fragment')(data)[0]
        self._child_map=e._child_map
        self._child_ids=e._child_ids
        self._properties=e._properties
        for k,v in e.propertyItems():
            setattr(self, k, v)
            
    manage_editForm=HTMLFile('elementEdit', globals())
        
        
# DOM Builder class
# -----------------

class DOMBuilder(Builder.Builder):
    nodes={
        ZDOM.DOCUMENT_NODE : Document,
        ZDOM.DOCUMENT_FRAGMENT_NODE : DocumentFragment,
        ZDOM.ELEMENT_NODE : Element,
        ZDOM.TEXT_NODE : Text,
        ZDOM.CDATA_SECTION_NODE : CDATASection,
        ZDOM.COMMENT_NODE : Comment,
        ZDOM.PROCESSING_INSTRUCTION_NODE : ProcessingInstruction,
        ZDOM.ATTRIBUTE_NODE : ZDOM.Attr,
        }
        
# Add form and method
# -------------------

addForm=HTMLFile('documentAdd', globals())

def add(self, id, title='', file='', REQUEST=None, submit=None):
    """
    Add a XML Document object with the contents of file.
    """
    ob=Document()
    ob.title=title
    ob.id=id
    if file: ob.parse(file)
    id=self._setObject(id, ob)
    if REQUEST is not None:
        try: u=self.DestinationURL()
        except: u=REQUEST['URL1']
        if submit==" Add and Edit ": u="%s/%s" % (u,quote(id))
        REQUEST.RESPONSE.redirect(u+'/manage_main')
    return ''
